using System;

namespace Utils
{
    /**
     * Implementation based on algorithm idea by:
     * Joshua Bloch, Effective Java Programming Language Guide, Prentice Hall, 2001
     */
    public class HashCodeBuilder
    {
        private readonly int constant;
        private int result;

        public HashCodeBuilder()
        {
            constant = 37;
            result = 17;
        }
        public HashCodeBuilder(int constant, int result)
        {
            if (constant % 2 == 0)
            {
                throw new ArgumentOutOfRangeException("HashCodeBuilder: Parameter 'constant' must be an odd value.");
            }
            this.constant = constant;
            this.result = result;
        }

        public int HashCode
        {
            get
            {
                return result;
            }
        }

        public HashCodeBuilder AppendSuper(int hashCode)
        {
            result = constant * result + hashCode;
            return this;
        }

        public HashCodeBuilder Append(bool f)
        {
            result = constant * result + (f ? 0 : 1);
            return this;
        }
        public HashCodeBuilder Append(byte f)
        {
            result = constant * result + (int)f;
            return this;
        }
        public HashCodeBuilder Append(char f)
        {
            result = constant * result + (int)f;
            return this;
        }
        public HashCodeBuilder Append(short f)
        {
            result = constant * result + (int)f;
            return this;
        }
        public HashCodeBuilder Append(int f)
        {
            result = constant * result + f;
            return this;
        }
        public HashCodeBuilder Append(long f)
        {
            result = constant * result + (int)(f ^ (f >> 32));
            return this;
        }
        public HashCodeBuilder Append(float f)
        {
            result = constant * result + Convert.ToInt32(f);
            return this;
        }
        public HashCodeBuilder Append(double f)
        {
            return Append(BitConverter.DoubleToInt64Bits(f));
        }

        public HashCodeBuilder Append(Object f)
        {
            if (f != null)
            {
                if (f.GetType().IsArray == false)
                {
                    result = constant * result + f.GetHashCode();
                }
                else
                {
                    if (f is bool[])
                    {
                        Append((bool[])f);
                    }
                    else if (f is byte[])
                    {
                        Append((byte[])f);
                    }
                    else if (f is char[])
                    {
                        Append((char[])f);
                    }
                    else if (f is short[])
                    {
                        Append((short[])f);
                    }
                    else if (f is int[])
                    {
                        Append((int[])f);
                    }
                    else if (f is long[])
                    {
                        Append((long[])f);
                    }
                    else if (f is float[])
                    {
                        Append((float[])f);
                    }
                    else if (f is double[])
                    {
                        Append((double[])f);
                    }
                    else
                    {
                        Append((Object[])f);
                    }
                }
            }
            return this;
        }

        public HashCodeBuilder Append(bool[] f)
        {
            if (f != null)
            {
                for (int i = 0; i < f.Length; i++)
                {
                    Append(f[i]);
                }
            }
            return this;
        }
        public HashCodeBuilder Append(byte[] f)
        {
            if (f != null)
            {
                for (int i = 0; i < f.Length; i++)
                {
                    Append(f[i]);
                }
            }
            return this;
        }
        public HashCodeBuilder Append(char[] f)
        {
            if (f != null)
            {
                for (int i = 0; i < f.Length; i++)
                {
                    Append(f[i]);
                }
            }
            return this;
        }
        public HashCodeBuilder Append(short[] f)
        {
            if (f != null)
            {
                for (int i = 0; i < f.Length; i++)
                {
                    Append(f[i]);
                }
            }
            return this;
        }
        public HashCodeBuilder Append(int[] f)
        {
            if (f != null)
            {
                for (int i = 0; i < f.Length; i++)
                {
                    Append(f[i]);
                }
            }
            return this;
        }
        public HashCodeBuilder Append(long[] f)
        {
            if (f != null)
            {
                for (int i = 0; i < f.Length; i++)
                {
                    Append(f[i]);
                }
            }
            return this;
        }
        public HashCodeBuilder Append(float[] f)
        {
            if (f != null)
            {
                for (int i = 0; i < f.Length; i++)
                {
                    Append(f[i]);
                }
            }
            return this;
        }
        public HashCodeBuilder Append(double[] f)
        {
            if (f != null)
            {
                for (int i = 0; i < f.Length; i++)
                {
                    Append(f[i]);
                }
            }
            return this;
        }
        public HashCodeBuilder Append(Object[] f)
        {
            if (f != null)
            {
                for (int i = 0; i < f.Length; i++)
                {
                    Append(f[i]);
                }
            }
            return this;
        }
    }
}
